
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#if HAVE_CONFIG_H
#include <config.h>
#endif

#include "gspell-enum-types.h"
/* enumerations from "gspell-checker.h" */
#include "gspell-checker.h"
GType
gspell_checker_error_get_type (void)
{
  static GType the_type = 0;

  if (the_type == 0)
    {
      static const GEnumValue values[] = {
        { GSPELL_CHECKER_ERROR_DICTIONARY,
          "GSPELL_CHECKER_ERROR_DICTIONARY",
          "dictionary" },
        { GSPELL_CHECKER_ERROR_NO_LANGUAGE_SET,
          "GSPELL_CHECKER_ERROR_NO_LANGUAGE_SET",
          "no-language-set" },
        { 0, NULL, NULL }
      };

      the_type = g_enum_register_static (
        g_intern_static_string ("GspellCheckerError"),
        values);
    }

  return the_type;
}

/* Generated data ends here */

